/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ws.integration.jboss50;

//$Id: JAXRPCDeployerJSE.java 2641 2007-03-19 20:06:28Z heiko.braun@jboss.com $

import org.jboss.deployers.spi.DeploymentException;
import org.jboss.deployers.spi.deployer.DeploymentUnit;
import org.jboss.metadata.WebMetaData;
import org.jboss.virtual.VirtualFile;
import org.jboss.ws.core.server.UnifiedDeploymentInfo;
import org.jboss.ws.core.server.UnifiedDeploymentInfo.DeploymentType;
import org.jboss.ws.core.utils.DOMUtils;
import org.jboss.ws.integration.UnifiedVirtualFile;
import org.jboss.ws.metadata.builder.jaxrpc.JAXRPCDeployment;
import org.w3c.dom.Element;

import java.io.IOException;

/**
 * A deployer JAXRPC JSE Endpoints
 *
 * @author Thomas.Diesler@jboss.org
 * @since 31-Oct-2005
 */
public class JAXRPCDeployerJSE extends AbstractDeployerJSE
{
   private static final String WEBSERVICES_XML = "webservices.xml";

   @Override
   protected DeploymentType getDeploymentType()
   {
      return DeploymentType.JAXRPC_JSE;
   }

   @Override
   protected UnifiedDeploymentInfo createUnifiedDeploymentInfo(DeploymentUnit unit) throws DeploymentException
   {
      UnifiedVirtualFile vfWebservices = getWebServicesFile(unit);
      UnifiedDeploymentInfo udi = new JAXRPCDeployment(getDeploymentType(), vfWebservices);
      DeploymentInfoAdaptor.buildDeploymentInfo(udi, unit);
      return udi;
   }

   @Override
   public boolean isWebServiceDeployment(DeploymentUnit unit)
   {
      boolean isWebserviceDeployment = unit.getAllMetaData(WebMetaData.class).size() > 0;
      isWebserviceDeployment = (isWebserviceDeployment && getWebServicesFile(unit) != null);

      if(isWebserviceDeployment)
      {
         // verify the DD namespace
         try
         {
            Element root = DOMUtils.parse( unit.getMetaDataFile(WEBSERVICES_XML).openStream() );
            isWebserviceDeployment = (isWebserviceDeployment && root.getNamespaceURI().equals("http://java.sun.com/xml/ns/j2ee")); 
         }
         catch (IOException e)
         {
            throw new IllegalArgumentException("Failed to parse webservice.xml", e);
         }
      }

      return isWebserviceDeployment;
   }

   private UnifiedVirtualFile getWebServicesFile(DeploymentUnit unit)
   {
      VirtualFile vfile = unit.getMetaDataFile(WEBSERVICES_XML);
      return vfile != null ? new VirtualFileAdaptor(vfile) : null;
   }
}
